import puppeteer, { product } from "puppeteer";
import jsonfile from "jsonfile";
import cheerio from "cheerio";
import fs from "fs-extra";
import { getDomain } from "tldts";

// Import website modules
import { 
    FitnessKoerier,
    FitnessApparaat,
    BolCom,
    BeterSport,
    FitnessGeest,
    BodyAndFit
} from "./modules/websites";

start();

// Globals
let browser: puppeteer.Browser = null;
let allProducts: Array<any> = [];
let allBlogUrls: Array<any> = [];

async function start() {
    try {
        console.log(`---- ContentEggStockCrawler ----`);
        
        console.log(`Starting crawler browser..`);
        browser = await puppeteer.launch({ headless: true });

        const urls = await jsonfile.readFile("./urls-check.json");
        console.log(`Loaded ${urls.length} urls from file! (urls-check.json)`);

        console.log(`Initialization done!`);
        console.log(`------------------------------------- \n`);

        for (let url of urls) {
            allBlogUrls.push(url);
            await crawlBlogPage(url);
        }

        await browser.close();
        await generateReport();
    } catch (error) {
        console.error(`An Error Occured!`, error);    
    }
}

async function crawlBlogPage(url: string) {
    try {
        console.log(`Crawling blog page: ${url}`);
    
        // Open new page and goto url
        const page = await browser.newPage();
        await page.goto(url, { waitUntil: 'networkidle2' });

        // Get html from page
        const html = await page.evaluate(() => document.body.innerHTML);

        // Close page
        await page.close();

        // Load html in cheerio object
        const $ = cheerio.load(html);

        let products: Array<any> = [];

        console.log(`Detected ${$(".row-products").length} content egg products!`);

        $(".row-products .cegg-list-logo-title a").each((index, element) => {
            let productUrl = $(element).attr("href");
            let name = $(element).html().trim();
            
            products.push({
                name: name,
                blogUrl: url,
                url: productUrl,
                domain: "",
                inStock: false,
            });
        });

        console.log("Checking product stocks...");

        for (let index in products) {
            let status = await crawlProductStock(products[index].url);
            
            products[index].domain = status[0]; 
            products[index].url = status[1];
            products[index].inStock = status[2];
           
            if (products[index].inStock) {
                console.log(`✅ ${products[index].name} - ${products[index].domain}`);
            } else {
                console.log(`❌ ${products[index].name} - ${products[index].domain}`);
            }

            allProducts.push(products[index]);
        }

        console.log("");
        console.log("");

    } catch (error) {
        console.log("Error while trying to crawl page! Skipping...");   
    }
}

async function crawlProductStock(url: string) {
    try {
        // Open new page and goto url
        const page = await browser.newPage();
        await page.goto(url, { waitUntil: 'networkidle2' });

        // Get html from page
        const html = await page.evaluate(() => document.body.innerHTML);

        // Load html in cheerio object
        const $ = cheerio.load(html);

        // Get domain from url
        const domain = getDomain(page.url());

        // Close page
        await page.close();

        switch (domain) {
            case "bol.com":
                return [domain, page.url(), await BolCom.check(html)];

            case "fitnesskoerier.nl":
                return [domain, page.url(), await FitnessKoerier.check(html)];
            
            case "fitnessapparaat.nl":
                return [domain, page.url(), await FitnessApparaat.check(html)];

            case "betersport.nl":
                return [domain, page.url(), await BeterSport.check(html)];

            case "fitness-geest.nl":
                return [domain, page.url(), await FitnessGeest.check(html)];
            
            case "bodyandfit.com":
                return [domain, page.url(), await BodyAndFit.check(html)];
            
            default:
                console.error(`-- ${domain} is not an available website module! Can't check stock!`);
                return ["NOT_MODULE", false];
        }
    } catch (error) {
        console.error(error);
        console.log("Error while trying to crawl page! Skipping...");
    }
}

async function generateReport() {
    console.log("Generating report...");
    
    let file = "./report.txt";
    let totalProducts = 0;
    let totalOutStock = 0;
    for (let product of allProducts) {
        totalProducts++;

        if (product.inStock == false) {
            totalOutStock++;
        }
    }

    await fs.appendFile(file, `----------------------------------------\n`);
    await fs.appendFile(file, `Content Egg Product Stock Crawler Report\n`);
    await fs.appendFile(file, `Total Blog Urls: ${allBlogUrls.length}\n`);
    await fs.appendFile(file, `Total Products: ${totalProducts}\n`);
    await fs.appendFile(file, `Total Out of Stock: ${totalOutStock}\n`);
    await fs.appendFile(file, `----------------------------------------\n`);
    await fs.appendFile(file, ``);

    let lastBlogUrl = "";
    for (let product of allProducts) {
        if (product.blogUrl != lastBlogUrl) {
            await fs.appendFile(file, `\n\n`);
            await fs.appendFile(file, `${product.blogUrl}\n`);
            await fs.appendFile(file, `----------------------------------\n`);
        }

        if (product.inStock == false) {
            await fs.appendFile(file, `${product.domain} - ${product.name} - ${product.url}\n`);
        }

        lastBlogUrl = product.blogUrl;
    }

    console.log("Report generated!");
}

